@@ -31,8 +31,7 @@
 struct output_buffer_t
 {
   output_buffer_t (option_parser_t *parser)
-		  : options (parser,
-			     g_strjoinv ("/", (gchar**) hb_buffer_serialize_list_formats ())),
+		  : options (parser, hb_buffer_serialize_list_formats ()),
 		    format (parser),
 		    gs (NULL),
 		    line_no (0),
@@ -53,14 +52,15 @@ struct output_buffer_t
     {
       if (options.explicit_output_format)
 	fail (false, "Unknown output format `%s'; supported formats are: %s",
-	      options.output_format, options.supported_formats);
+	      options.output_format,
+	      g_strjoinv ("/", const_cast<char**> (options.supported_formats)));
       else
 	/* Just default to TEXT if not explicitly requested and the
 	 * file extension is not recognized. */
 	output_format = HB_BUFFER_SERIALIZE_FORMAT_TEXT;
     }
 
-    unsigned int flags = HB_BUFFER_SERIALIZE_FLAGS_DEFAULT;
+    unsigned int flags = HB_BUFFER_SERIALIZE_FLAG_DEFAULT;
     if (!format.show_glyph_names)
       flags |= HB_BUFFER_SERIALIZE_FLAG_NO_GLYPH_NAMES;
     if (!format.show_clusters)
